@@ -20,6 +20,13 @@ This private submodule is *not* intended for importation by downstream callers.
 '''
 
 # ....................{ TODO                              }....................
+#FIXME: Major optimization: duplicate the signature of the decorated callable
+#as the signature of our wrapper function. Why? Because doing so obviates the
+#need to explicitly test whether each possible parameter was passed and how
+#that parameter was passed (e.g., positional, keyword) as well as the need to
+#localize "__beartype_args_len" and so on. In short, this is a massive win.
+#Again, see the third-party "makefun" package, which purports to already do so.
+
 #FIXME: Remove duplicates from tuple annotations for efficiency: e.g.,
 #
 #    # This...
@@ -117,6 +124,8 @@ This private submodule is *not* intended for importation by downstream callers.
 # ....................{ IMPORTS                           }....................
 from beartype.roar import BeartypeDecorParamNameException
 from beartype._decor._code._codesnip import (
+    CODE_INIT_PARAMS_POSITIONAL_LEN,
+    CODE_INIT_RANDOM_INT,
     CODE_RETURN_UNCHECKED,
     CODE_SIGNATURE,
 
@@ -260,6 +269,20 @@ def generate_code(data: BeartypeData) -> None:
     # callable if any *or* the empty string otherwise.
     code_return = _code_check_return(data)
 
+    # Python code snippet declaring the signature of this wrapper followed by
+    # preliminary statements (e.g., assignment initializations) if desired
+    # *AFTER* generating snippets type-checking parameters and return values,
+    # both of which modify instance variables of the dataclass tested below.
+    code_init = (
+        # If the body of this wrapper requires a pseudo-random integer, append
+        # code generating and localizing such an integer to this signature.
+        code_sig + CODE_INIT_RANDOM_INT
+        if data.is_func_wrapper_needs_random_int else
+        # Else, this body requires *NO* such integer. In this case, preserve
+        # this signature as is.
+        code_sig
+    )
+
     #FIXME: Refactor to leverage f-strings after dropping Python 3.5 support,
     #which are the optimal means of performing string formatting.
 
@@ -275,7 +298,7 @@ def generate_code(data: BeartypeData) -> None:
     #
     # Since string concatenation is heavily optimized by the official CPython
     # interpreter, the simplest approach is the most ideal.
-    func_code = code_sig + code_params + code_return
+    func_code = code_init + code_params + code_return
 
     # True only if this code proxies this callable *WITHOUT* type checking.
     is_func_code_noop = (func_code == code_sig + CODE_RETURN_UNCHECKED)
@@ -324,6 +347,9 @@ def _code_check_params(data: BeartypeData) -> str:
     # Python code snippet to be returned.
     func_code = ''
 
+    # True only if this callable accepts one or more positional parameters.
+    is_params_positional = False
+
     # For the name of each parameter accepted by this callable and the
     # "Parameter" instance encapsulating this parameter (in declaration
     # order)...
@@ -361,6 +387,12 @@ def _code_check_params(data: BeartypeData) -> str:
         if (func_arg.annotation in HINTS_IGNORABLE or
             func_arg.kind in _PARAM_KINDS_IGNORABLE):
             continue
+        # Else, this parameter is non-ignorable.
+        #
+        # If this non-ignorable parameter either may or must be passed
+        # positionally, note this fact.
+        elif func_arg.kind is Parameter.POSITIONAL_OR_KEYWORD:
+            is_params_positional = True
 
         # If this is a PEP-compliant hint, append Python code type-checking
         # this parameter against this hint.
@@ -379,8 +411,16 @@ def _code_check_params(data: BeartypeData) -> str:
                 func_arg_index=func_arg_index,
             )
 
-    # Return this snippet.
-    return func_code
+    # Return either...
+    return (
+        # If this callable accepts one or more positional parameters, this
+        # snippet preceded by code localizing the number of such parameters.
+        CODE_INIT_PARAMS_POSITIONAL_LEN + func_code
+        if is_params_positional else
+        # Else, this callable accepts *NO* positional parameters. In this case,
+        # this snippet as is.
+        func_code
+    )
 
 # ....................{ CODERS                            }....................
 def _code_check_return(data: BeartypeData) -> str:
