@@ -16,6 +16,9 @@ This private submodule is *not* intended for importation by downstream callers.
 '''
 
 # ....................{ TODO                              }....................
+#FIXME: Drop Python 3.5 support with the next minor beartype release (i.e.,
+#beartype 0.3.0), as Python 3.5 hit EOL on September 13th, 2020.
+
 #FIXME: Reduce @beartype to a noop if the passed callable was decorated by the
 #@typing.no_type_check decorator, which we can trivially detect at runtime via
 #the "__no_type_check__" dunder attribute set on that callable set to True.
@@ -54,20 +57,6 @@ This private submodule is *not* intended for importation by downstream callers.
 #Python 3.5 support. For now, the current approach is fine albeit slower than
 #ideal, which will be resolved shortly. *shrug*
 
-#FIXME: Avoid naively passing globals() to the exec() call below. Instead, we
-#should pass a new global constant containing all "__beartype_" global imports:
-#     #FIXME: Note that this definition is no longer definitive and will
-#     #probably require trivial revision to comply with modern imports above.
-#     _GLOBAL_ATTRS = {
-#         '__beartype_die_unless_hint_nonpep': die_unless_hint_nonpep,
-#         '__beartype_nonpep_param_exception': BeartypeCallCheckNonPepParamException,
-#         '__beartype_nonpep_return_exception': BeartypeCallCheckNonPepReturnException,
-#         '__beartype_trim': trim_object_repr,
-#     }
-#Then refactor that exec() call to resemble:
-#        exec(func_code, _GLOBAL_ATTRS, local_attrs)
-#Note this is mildly faster as well, as we avoid the globals() call.
-
 #FIXME: *CRITICAL EDGE CASE:* If the passed "func" is a coroutine, that
 #coroutine *MUST* be called preceded by the "await" keyword rather than merely
 #called as is. Detecting coroutines is trivial, thankfully: e.g.,
@@ -203,8 +192,11 @@ This private submodule is *not* intended for importation by downstream callers.
 #  thus never be the default.
 
 # ....................{ IMPORTS                           }....................
-import functools
+import functools, random
 from beartype.roar import (
+    BeartypeCallCheckNonPepParamException,
+    BeartypeCallCheckNonPepReturnException,
+    BeartypeCallCheckPepException,
     BeartypeDecorWrappeeException,
     BeartypeDecorWrapperException,
 )
@@ -217,39 +209,48 @@ from beartype._decor._data import BeartypeData
 from beartype._decor._typistry import bear_typistry
 from beartype._util.cache.pool.utilcachepoolobjecttyped import (
     acquire_object_typed, release_object_typed)
+from beartype._util.hint.nonpep.utilhintnonpeptest import (
+    die_unless_hint_nonpep)
+from beartype._util.hint.pep.error.utilhintpeperror import (
+    raise_pep_call_exception)
+from beartype._util.text.utiltextmunge import number_lines
+from beartype._util.text.utiltextrepr import get_object_representation
+# from types import FunctionType
 
 # See the "beartype.__init__" submodule for further commentary.
 __all__ = ['STAR_IMPORTS_CONSIDERED_HARMFUL']
 
-# ....................{ IMPORTS ~ wrapper                 }....................
-# Attributes required by callables generated by the @beartype decorator and
-# hence embedded in one or more string global constants declared by the
-# "snippet" submodule. For uniqueness, these attributes are imported under
-# alternate names prefixed by "__beartype_".
-#
-# Note that these attributes are intentionally imported at module scope and
-# thus implicitly available to these callables via the "globals()" dictionary
-# passed to the exec() builtin called by the @beartype decorator. Technically,
-# these attributes could also be locally passed in that decorator to those
-# callables by adding these attributes to the "local_attrs" dictionary and then
-# explicitly passing each such attribute as a unique keyword parameter to those
-# callables. Since doing so would uselessly incur a runtime performance penalty
-# for no tangible gain, the current approach is preferable.
-from beartype.roar import (
-    BeartypeCallCheckNonPepParamException  as __beartype_nonpep_param_exception,
-    BeartypeCallCheckNonPepReturnException as __beartype_nonpep_return_exception,
-    BeartypeCallCheckPepException          as __beartype_pep_exception,
-)
-from beartype._util.hint.pep.utilhintpepcall import (
-    raise_pep_call_exception as __beartype_raise_pep_call_exception,
-)
-from beartype._util.hint.nonpep.utilhintnonpeptest import (
-    die_unless_hint_nonpep as __beartype_die_unless_hint_nonpep,
-)
-from beartype._util.text.utiltextmunge import (
-    trim_object_repr as __beartype_trim,
-    number_lines,
-)
+# ....................{ CONSTANTS                         }....................
+_GLOBAL_ATTRS = {
+    '__beartype_die_unless_hint_nonpep': die_unless_hint_nonpep,
+    '__beartype_getrandbits': random.getrandbits,
+    '__beartype_nonpep_param_exception': (
+        BeartypeCallCheckNonPepParamException),
+    '__beartype_nonpep_return_exception': (
+        BeartypeCallCheckNonPepReturnException),
+    '__beartype_pep_exception': BeartypeCallCheckPepException,
+    '__beartype_raise_pep_call_exception': raise_pep_call_exception,
+    '__beartype_trim': get_object_representation,
+}
+'''
+Dictionary mapping from the name to value of all attributes internally
+accessed as globals (rather than as locals externally passed as private default
+parameters) in wrapping functions created and returned by the :func:`beartype`
+decorator.
+
+The names of these attributes are embedded in one or more string global
+constants declared by one or more snippet submodules (e.g.,
+:mod:`beartype._decor._code._codesnip`). To avoid colliding with the names of
+arbitrary caller-defined parameters, these names *must* be aliased under
+alternate names prefixed by ``__beartype_``.
+
+Caveats
+----------
+**Attributes frequently accessed in the body of these functions should instead
+be externally passed as default parameters into these functions.** This
+includes the frequently accessed ``__beartypistry`` local, which is thus passed
+as a private default parameter to the signatures of these functions.
+'''
 
 # ....................{ DECORATORS                        }....................
 def beartype(func):
@@ -304,7 +305,7 @@ def beartype(func):
     BeartypeDecorParamNameException
         If the name of any parameter declared on this callable is prefixed by
         the reserved substring ``__beartype_``.
-    BeartypeDecorPep563Exception
+    BeartypeDecorHintPep563Exception
         If `PEP 563`_ is active for this callable and evaluating a **postponed
         annotation** (i.e., annotation whose value is a string) on this
         callable raises an exception (e.g., due to that annotation referring to
@@ -379,7 +380,7 @@ def beartype(func):
     # * For efficiency, only attributes specific to the body of this wrapper
     #   are copied from the current namespace. Attributes generically
     #   applicable to the body of all wrappers are instead implicitly imported
-    #   from this submodule by passing "globals()" below.
+    #   from this submodule by passing "_GLOBAL_ATTRS" below.
     # * For each attribute specified here, one new keyword parameter of the
     #   form "{local_attr_key_name}={local_attr_key_name}" *MUST* be added to
     #   the signature for this wrapper defined by the "CODE_SIGNATURE" string.
@@ -391,6 +392,36 @@ def beartype(func):
         PARAM_NAME_TYPISTRY: bear_typistry,
     }
 
+    #FIXME: Refactor to leverage f-strings after dropping Python 3.5 support,
+    #which are the optimal means of performing string formatting.
+
+    #FIXME: Uncomment after unit testing
+    #utilcallable.get_callable_filename_or_placeholder() up.
+    #FIXME: Once this is working, use the commented code example starting with
+    #"func_code_compiled = compile" given below to associate this filename with
+    #this wrapper function.
+
+    # Fake filename of the in-memory fake file masquerading as declaring this
+    # wrapper function. This filename guarantees the uniqueness of the 3-tuple
+    # ``({func_filename}, {func_file_line_number}, {func_name})`` containing
+    # this filenames commonly leveraged by profilers (e.g., "cProfile") to
+    # identify arbitrary callables, where:
+    # * `{func_filename}` is this filename (e.g.,
+    #   `"</home/leycec/py/betse/betse/lib/libs.py:beartype({func_name})>"`).
+    # * `{func_file_line_number}`, is *ALWAYS* 0 and thus *NEVER* unique.
+    # * `{func_name}`, is identical to that of the decorated callable and also
+    #   thus *NEVER* unique.
+    #
+    # Ergo, uniquifying this filename is the *ONLY* means of uniquifying
+    # metadata identifying this wrapper function via runtime inspection.
+    #func_wrapper_filename = (
+    #    '<' +
+    #    get_callable_filename_or_placeholder(func) +
+    #    ':beartype(' +
+    #    func_data.func_name +
+    #    ')>'
+    #)
+
     #FIXME: Actually, we absolutely *DO* want to leverage the example
     #documented below of leveraging the compile() builtin. We want to do so
     #explicitly to pass something other than "<string>" here -- ideally,
@@ -419,6 +450,8 @@ def beartype(func):
     #preserving metadata on the original callable into our wrapper callable.
     #While we absolutely should *NOT* depend on that or any other third-party
     #package, that package's implementation should lend us useful insight.
+    #Indeed, see the _make() function of the "makefun.main" submodule:
+    #    https://github.com/smarie/python-makefun/blob/master/makefun/main.py
 
     # Attempt to define this wrapper as a closure of this decorator. For
     # obscure and presumably uninteresting reasons, Python fails to locally
@@ -432,7 +465,7 @@ def beartype(func):
     #         func_code, "<string>", "exec").co_consts[0]
     #     return types.FunctionType(
     #         code=func_code_compiled,
-    #         globals=globals(),
+    #         globals=_GLOBAL_ATTRS,
     #         argdefs=('__beartype_func', func)
     #     )
     #
@@ -440,7 +473,20 @@ def beartype(func):
     # the current circumspect approach is preferred.
     try:
         # print('\n@beartyped {}() wrapper\n\n{}\n'.format(func_data.func_name, func_code))
-        exec(func_code, globals(), local_attrs)
+        exec(func_code, _GLOBAL_ATTRS, local_attrs)
+
+        #FIXME: See above.
+        #FIXME: Should "exec" be "single" instead? Does it matter? Is there any
+        #performance gap between the two?
+        # func_code_compiled = compile(
+        #     func_code, func_wrapper_filename, "exec").co_consts[0]
+        # return FunctionType(
+        #     code=func_code_compiled,
+        #     globals=_GLOBAL_ATTRS,
+        #
+        #     #FIXME: This really doesn't seem right, but... *shrug*
+        #     argdefs=tuple(local_attrs.values()),
+        # )
     # If doing so fails for any reason...
     except Exception as exception:
         # Debuggable wrapper code such that each line of this code is prefixed
