@@ -186,7 +186,7 @@ if _sys.version_info[:3] < PYTHON_VERSION_MIN_PARTS:
 # For further details, see http://semver.org.
 #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 
-VERSION = '0.2.0'
+VERSION = '0.3.0'
 '''
 Human-readable package version as a ``.``-delimited string.
 '''
@@ -241,6 +241,12 @@ have yet to be stabilized and hence tagged. Hence, this URL is typically valid
 *only* for previously released (rather than live) versions of this package.
 '''
 
+
+URL_ISSUES = 'https://github.com/beartype/beartype/issues'
+'''
+URL of this package's issue tracker.
+'''
+
 # ....................{ METADATA ~ libs                   }....................
 LIBS_RUNTIME_OPTIONAL = ()
 '''
